@@ -12,6 +12,7 @@ from collections import namedtuple, OrderedDict
 # ===================================================================
 
 import crds
+import logging
 from crds.core import log, config, utils, timestamp, cmdline, heavy_client
 from crds import diff, matches
 from . import table_effects, headers
@@ -25,13 +26,14 @@ MAX_DATE = "9999-01-01 23:59:59"
 # ===================================================================
 
 UpdateTuple = namedtuple("UpdateTuple", ["instrument", "filekind", "old_reference", "new_reference"])
+LOGGER = logging.getLogger(__name__)
 
 # ============================================================================
 
 
 def reformat_date_or_auto(date):
     """Add 'auto' as an extra valid value for --datasets-since dates.  Auto means figure out dates-since
-    based on USEAFTER dates (recorded in rmaps as DATE-OBS TIME-OBS or META.OBSERVATION.DATE).
+    based on USEAFTER dates (recorded in rmaps as DATE-OBS TIME-OBS, META.OBSERVATION.DATE, or META.EXPOSURE.START_TIME).
     """
     if date is None:
         return date
