@@ -8,6 +8,7 @@ import base64
 import re
 import zlib
 import html
+import importlib.metadata
 from urllib import request
 import warnings
 import json
@@ -33,6 +34,7 @@ from .proxy import CheckingProxy
 __all__ = [
     "get_default_observatory",
     "get_default_context",
+    "get_build_context",
     "get_context_by_date",
     "get_server_info",
     "get_cached_server_info",  # deprecated
@@ -44,7 +46,7 @@ __all__ = [
     "list_mappings",
     "list_references",
 
-    "get_url",      # deprecated
+    "get_url",  # deprecated
     "get_flex_uri",
     "get_file_info",
     "get_file_info_map",
@@ -78,7 +80,7 @@ __all__ = [
     "jpoll_abort",
 
     "get_system_versions",
-    ]
+]
 
 # ============================================================================
 
@@ -86,7 +88,8 @@ __all__ = [
 
 URL_SUFFIX = "/json/"
 
-S = None    # Proxy server
+S = None  # Proxy server
+
 
 def set_crds_server(url):
     """Configure the CRDS JSON services server to `url`,
@@ -100,6 +103,7 @@ def set_crds_server(url):
     URL = url + URL_SUFFIX
     S = CheckingProxy(URL, version="1.0")
 
+
 def get_crds_server():
     """Return the base URL for the CRDS JSON RPC server.
     """
@@ -108,8 +112,10 @@ def get_crds_server():
         log.warning("CRDS_SERVER_URL does not start with https://  ::", url)
     return url
 
+
 # =============================================================================
 
+
 @utils.cached
 def list_mappings(observatory=None, glob_pattern="*"):
     """Return the list of mappings associated with `observatory`
@@ -117,6 +123,7 @@ def list_mappings(observatory=None, glob_pattern="*"):
     """
     return [str(x) for x in S.list_mappings(observatory, glob_pattern)]
 
+
 @utils.cached
 def list_references(observatory=None, glob_pattern="*"):
     """Return the list of references associated with `observatory`
@@ -124,6 +131,7 @@ def list_references(observatory=None, glob_pattern="*"):
     """
     return [str(x) for x in S.list_references(observatory, glob_pattern)]
 
+
 def get_mapping_url(pipeline_context, mapping):
     """Returns a URL for the specified pmap, imap, or rmap file.   DEPRECATED
     """
@@ -131,6 +139,7 @@ def get_mapping_url(pipeline_context, mapping):
         "crds.client.get_mapping_url()", "2020-09-01", "crds.client.get_flex_uri()")
     return S.get_mapping_url(pipeline_context, mapping)
 
+
 def is_known_mapping(mapping):
     """Return True iff `mapping` is a known/official CRDS mapping file."""
     try:
@@ -138,6 +147,7 @@ def is_known_mapping(mapping):
     except ServiceError:
         return False
 
+
 @utils.cached
 def get_mapping_names(pipeline_context):
     """Get the complete set of pmap, imap, and rmap basenames required
@@ -146,6 +156,7 @@ def get_mapping_names(pipeline_context):
     """
     return [str(x) for x in S.get_mapping_names(pipeline_context)]
 
+
 def get_reference_url(pipeline_context, reference):
     """Returns a URL for the specified reference file.    DEPRECATED
     """
@@ -153,6 +164,7 @@ def get_reference_url(pipeline_context, reference):
         "crds.client.get_reference_url()", "2020-09-01", "crds.client.get_flex_uri()")
     return S.get_reference_url(pipeline_context, reference)
 
+
 def get_url(pipeline_context, filename):
     """Return the URL for a CRDS reference or mapping file.   DEPRECATED
     """
@@ -160,6 +172,7 @@ def get_url(pipeline_context, filename):
         "crds.client.get_url()", "2020-09-01", "crds.client.get_flex_uri()")
     return S.get_url(pipeline_context, filename)
 
+
 def get_flex_uri(filename, observatory=None):
     """If environment variables define the base URI for `filename`, append
     filename and return the combined URI.
@@ -192,20 +205,23 @@ def get_flex_uri(filename, observatory=None):
         uri += filename
     return uri
 
+
 def _unpack_info(info, section, observatory):
     """Return info[section][observatory] if info[section] is defined.
     Otherwise return "none".
     """
     sect = info.get(section)
     if sect:
-       return sect[observatory]
+        return sect[observatory]
     else:
         return "none"
 
+
 def get_file_info(pipeline_context, filename):
     """Return a dictionary of CRDS information about `filename`."""
     return S.get_file_info(pipeline_context, filename)
 
+
 def get_file_info_map(observatory, files=None, fields=None):
     """Return the info { filename : { info } } on `files` of `observatory`.
     `fields` can be used to limit info returned to specified keys.
@@ -216,12 +232,39 @@ def get_file_info_map(observatory, files=None, fields=None):
         fields = tuple(sorted(fields))
     return _get_file_info_map(observatory, files, fields)
 
+
 @utils.cached
 def _get_file_info_map(observatory, files, fields):
     """Memory cached version of get_file_info_map() service."""
     infos = S.get_file_info_map(observatory, files, fields)
     return infos
 
+
+def get_cal_dist_path(cal):
+    try:
+        return f' ({str(importlib.metadata.distribution(cal)._path)})'
+    except Exception:
+        return ''
+
+
+def get_cal_version(observatory):
+    """Return the version of observatory calibration software."""
+    cal_version = ''
+    if observatory:
+        cal = dict(jwst='jwst', roman='romancal', hst='caldp')[observatory]
+        try:
+            cal_version = importlib.metadata.version(cal)
+            if 'dev' in cal_version:
+                log.info("DEV calibration SW identified. Defaulting to Edit Context.")
+                return 'dev'
+            cal_version = config.simplify_version(cal_version)
+            dist_path = get_cal_dist_path(cal)
+            log.info(f"Calibration SW Found: {cal} {cal_version}{dist_path}")
+        except importlib.metadata.PackageNotFoundError:
+            log.warning("Calibration SW not found, defaulting to latest.")
+    return cal_version
+
+
 def get_total_bytes(info_map):
     """Return the total byte count of file info map `info_map`."""
     try:
@@ -230,9 +273,10 @@ def get_total_bytes(info_map):
         log.error("Error computing total byte count: ", str(exc))
         return -1
 
+
 def get_sqlite_db(observatory):
     """Download the CRDS database as a SQLite database."""
-    assert not config.get_cache_readonly(), "Readonly cache, updating the SQLite database cannot be done."""
+    assert not config.get_cache_readonly(), "Readonly cache, updating the SQLite database cannot be done."
     encoded_compressed_data = S.get_sqlite_db(observatory)
     data = zlib.decompress(base64.b64decode(encoded_compressed_data))
     path = config.get_sqlite3_db_path(observatory)
@@ -241,6 +285,7 @@ def get_sqlite_db(observatory):
         db_out.write(data)
     return path
 
+
 @utils.cached
 def get_reference_names(pipeline_context):
     """Get the complete set of reference file basenames required
@@ -248,6 +293,7 @@ def get_reference_names(pipeline_context):
     """
     return [str(x) for x in S.get_reference_names(pipeline_context)]
 
+
 def get_best_references(pipeline_context, header, reftypes=None):
     """Get best references for dict-like `header` relative to
     `pipeline_context`.
@@ -261,13 +307,14 @@ def get_best_references(pipeline_context, header, reftypes=None):
 
     Raises           CrdsLookupError,  typically for problems with header values
     """
-    header = { str(key):str(value) for (key,value) in header.items() }
+    header = {str(key): str(value) for (key, value) in header.items()}
     try:
         bestrefs = S.get_best_references(pipeline_context, dict(header), reftypes)
     except Exception as exc:
         raise CrdsLookupError(str(exc)) from exc
     return bestrefs
 
+
 def get_best_references_by_ids(context, dataset_ids, reftypes=None, include_headers=False):
     """Get best references for the specified `dataset_ids` and reference types.  If
     reftypes is None,  all types are returned.
@@ -280,6 +327,7 @@ def get_best_references_by_ids(context, dataset_ids, reftypes=None, include_head
         raise CrdsLookupError(str(exc)) from exc
     return bestrefs
 
+
 def get_best_references_by_header_map(context, header_map, reftypes=None):
     """Get best references for header_map = { dataset_id : header, ...}, } and reference types
     where a header is a dictionary of matching parameters.
@@ -307,23 +355,77 @@ def get_aui_best_references(date, dataset_ids):
         raise CrdsLookupError(str(exc)) from exc
     return bestrefs_map
 
+
 @utils.cached
-def get_default_context(observatory=None):
-    """Return the name of the latest pipeline mapping in use for processing
-    files for `observatory`.
+def get_default_context(observatory=None, state=None):
+    """Return the name of the pipeline mapping ('.pmap') in use for processing
+    files for `observatory`. If `state` is None, for JWST this defaults to the build context
+    associated with the locally installed calibration software (cal_ver). For other missions
+    this defaults to `latest` (formerly `operational`).
+
+    Parameters
+    ----------
+    observatory : str, optional
+        observatory being used by current configuration, by default None
+    state : str, optional
+        context state ("latest", "build", "edit"), by default None
+
+    Returns
+    -------
+    str
+        name of the pipeline mapping ('.pmap') used to process files for `observatory`
     """
-    return str(S.get_default_context(observatory))
+    observatory = get_default_observatory() if observatory is None else observatory
+    if state == "build" or (observatory == "jwst" and state not in ["edit", "latest"]):
+        return get_build_context(observatory=observatory)
+    try:
+        return str(S.get_default_context(observatory, state))
+    except ServiceError: # backwards-compatibility for crds_server < 13.0.0
+        return str(S.get_default_context(observatory))
+
+
+def get_build_context(observatory=None):
+    """If available, return the name of the build context pipeline mapping in use for processing
+    files for `observatory`. Initially only planned use is for jwst but other mission
+    calibration pipeline sw is included as a template. If exact match is not found, an attempt to
+    find next closest (previous) patch version is made. Ultimate fallback is to the latest
+    (formerly 'operational') context.
+
+    Parameters
+    ----------
+    observatory : str, optional
+        observatory being used by current configuration, by default None
+
+    Returns
+    -------
+    str
+        name of the pipeline mapping ('.pmap') used to process files for `observatory` according to 
+        locally installed calibration software version.
+    """
+    observatory = get_default_observatory() if observatory is None else observatory
+    calver = get_cal_version(observatory)
+    if calver:
+        if calver == 'dev':
+            return get_default_context(observatory=observatory, state='edit')
+        else:
+            try:
+                return str(S.get_build_context(observatory, calver))
+            except ServiceError:
+                log.info("Server build context could not be identified. Using 'latest' instead.")
+    return get_default_context(observatory=observatory, state="latest")
+
 
 @utils.cached
 def get_context_by_date(date, observatory=None):
-    """Return the name of the first operational context which precedes `date`."""
+    """Return the name of the first latest context which precedes `date`."""
     return str(S.get_context_by_date(date, observatory))
 
+
 @utils.cached
 def get_server_info():
     """Return a dictionary of critical parameters about the server such as:
 
-    operational_context  - the context in use in the operational pipeline
+    latest_context  - the latest context in use on the server
 
     edit_context         - the context which was last edited, not
                            necessarily archived or operational yet.
@@ -361,12 +463,14 @@ def get_server_info():
         info["download_metadata"] = proxy.crds_encode(metadata)
     return info
 
+
 @utils.cached
 def get_download_metadata():
     "Defer and cache decoding of download_metadata field of server info."""
     info = get_server_info()
     return proxy.crds_decode(info["download_metadata"])
 
+
 def _get_server_info():
     """Fetch the server info dict.   If CRDS_CONFIG_URI is set then
     download that URL and load json from the contents.  Otherwise,
@@ -394,23 +498,28 @@ def _get_server_info():
             srepr(exc)) from exc
     return info
 
+
 get_cached_server_info = get_server_info
 
+
 def get_server_version():
     """Return the API version of the current CRDS server."""
     info = get_server_info()
     return info["crds_version"]["str"]
 
+
 def get_dataset_headers_by_id(context, dataset_ids, datasets_since=None):
     """Return { dataset_id : { header } } for `dataset_ids`."""
     context = os.path.basename(context)
     return S.get_dataset_headers_by_id(context, dataset_ids, datasets_since)
 
+
 def get_dataset_ids(context, instrument, datasets_since=None):
     """Return [ dataset_id, ...] for `instrument`."""
     context = os.path.basename(context)
     return S.get_dataset_ids(context, instrument, datasets_since)
 
+
 @utils.cached
 def get_required_parkeys(context):
     """Return a mapping from instruments to lists of parameter names required to
@@ -421,12 +530,14 @@ def get_required_parkeys(context):
     context = os.path.basename(context)
     return S.get_required_parkeys(context)
 
+
 def get_dataset_headers_by_instrument(context, instrument, datasets_since=None):
     """return { dataset_id:header, ...} for every `dataset_id` for `instrument`."""
     log.verbose("Dumping datasets for", repr(instrument))
     ids = get_dataset_ids(context, instrument, datasets_since)
     return dict(get_dataset_headers_unlimited(context, ids))
 
+
 def get_dataset_headers_unlimited(context, ids):
     """Generate (dataset_id, header) for `ids`,  potentially more
     `ids` than can be serviced with a single JSONRPC request.
@@ -435,16 +546,18 @@ def get_dataset_headers_unlimited(context, ids):
     """
     max_ids_per_rpc = get_server_info().get("max_headers_per_rpc", 500)
     for i in range(0, len(ids), max_ids_per_rpc):
-        log.verbose("Dumping dataset headers", i , "of", len(ids), verbosity=20)
-        id_slice = ids[i : i + max_ids_per_rpc]
+        log.verbose("Dumping dataset headers", i, "of", len(ids), verbosity=20)
+        id_slice = ids[i: i + max_ids_per_rpc]
         header_slice = get_dataset_headers_by_id(context, id_slice)
         for item in header_slice.items():
             yield item
 
+
 def get_affected_datasets(observatory, old_context=None, new_context=None):
     """Return a structure describing the ids affected by the last context change."""
     return utils.Struct(S.get_affected_datasets(observatory, old_context, new_context))
 
+
 def get_context_history(observatory):
     """Fetch the history of context transitions, a list of history era tuples:
 
@@ -452,22 +565,31 @@ def get_context_history(observatory):
     """
     return sorted(tuple(x) for x in S.get_context_history(observatory))
 
+
 def push_remote_context(observatory, kind, key, context):
-    """Upload the specified `context` of type `kind` (e.g. "operational") to the
-    server,  informing the server of the actual configuration of the local cache
-    for critical systems like pipelines,  not average users.   This lets the server
-    display actual versus commanded (Set Context) operational contexts for a pipeline.
+    """Upload the specified `context` of type `kind` (e.g. "latest") to the
+    server,  informing the server of the actual configuration of the local cache.   
+    This lets the server display actual versus commanded (Set Context) latest/operational contexts.
     """
     try:
         return S.push_remote_context(observatory, kind, key, context)
     except Exception as exc:
-        raise CrdsRemoteContextError(
-            "Server error setting pipeline context",
-            (observatory, kind, key, context)) from exc
+        if kind == 'operational':
+            try:
+                return S.push_remote_context(observatory, 'latest', key, context)
+            except Exception as exc:
+                raise CrdsRemoteContextError(
+                    "Server error setting latest context",
+                    (observatory, 'latest', key, context)) from exc
+        else:
+            raise CrdsRemoteContextError(
+                "Server error setting operational context",
+                (observatory, kind, key, context)) from exc
+
 
 def get_remote_context(observatory, pipeline_name):
     """Get the name of the default context last pushed from `pipeline_name` and
-    presumed to be operational.
+    presumed to be latest.
     """
     try:
         return S.get_remote_context(observatory, pipeline_name)
@@ -476,8 +598,10 @@ def get_remote_context(observatory, pipeline_name):
             "Server error resolving context in use by pipeline",
             (observatory, pipeline_name)) from exc
 
+
 # ==============================================================================
 
+
 def jpoll_pull_messages(key, since_id=None):
     """Return a list of jpoll json message objects from the channel associated
     with `key` sent after datetime string `since` or since the last pull if
@@ -490,12 +614,15 @@ def jpoll_pull_messages(key, since_id=None):
         messages.append(decoded)
     return messages
 
+
 def jpoll_abort(key):
     """Request that the process writing to jpoll terminate on its next write."""
     return S.jpoll_abort(key)
 
+
 # ==============================================================================
 
+
 def get_system_versions(master_version, context=None):
     """Return the versions Struct associated with cal s/w master_version as
     defined by `context` which can be defined as "null", "none", or None to use
@@ -503,10 +630,13 @@ def get_system_versions(master_version, context=None):
     """
     return utils.Struct(S.get_system_versions(master_version, str(context)))
 
+
 # ==============================================================================
 
+
 HARD_DEFAULT_OBS = "jwst"
 
+
 def get_server_observatory():
     """Return the default observatory according to the server, or None."""
     try:
@@ -517,6 +647,7 @@ def get_server_observatory():
         server_obs = observatory_from_string(pmap)
     return server_obs
 
+
 def get_default_observatory():
     """Based on the environment, cache, and server,  determine the default observatory.
 
@@ -526,11 +657,12 @@ def get_default_observatory():
     4. jwst
     """
     obs = config.OBSERVATORY.get()
-    if obs != "none":
+    if obs not in ["none", "", None]:
         return obs
     return observatory_from_string(get_crds_server()) or \
-           get_server_observatory() or \
-           "jwst"
+        get_server_observatory() or \
+        "jwst"
+
 
 def observatory_from_string(string):
     """If an observatory name is in `string`, return it,  otherwise return None."""
@@ -539,23 +671,28 @@ def observatory_from_string(string):
             return observatory
     return None
 
+
 # ==============================================================================
 
+
 def file_progress(activity, name, path, bytes, bytes_so_far, total_bytes, nth_file, total_files):
     """Output progress information for `activity` on file `name` at `path`."""
     return "{activity}  {path!s:<55}  {bytes} bytes  ({nth_file} / {total_files} files) ({bytes_so_far} / {total_bytes} bytes)".format(
         activity=activity,
         path=path,
         bytes=utils.human_format_number(bytes),
-        nth_file=nth_file+1,
+        nth_file=nth_file + 1,
         total_files=total_files,
         bytes_so_far=utils.human_format_number(bytes_so_far).strip(),
         total_bytes=utils.human_format_number(total_bytes).strip())
 
+
 # ==============================================================================
 
+
 class FileCacher:
     """FileCacher gets remote files with simple names into a local cache."""
+
     def __init__(self, pipeline_context, ignore_cache=False, raise_exceptions=True):
         self.pipeline_context = pipeline_context
         self.observatory = self.observatory_from_context()
@@ -576,7 +713,7 @@ class FileCacher:
         names2 = names[:]
         for refname in names2:
             if re.match(r"\w+\.r[0-9]h$", refname):
-                names.append(refname[:-1]+"d")
+                names.append(refname[:-1] + "d")
 
         downloads = []
         for name in names:
@@ -621,7 +758,8 @@ class FileCacher:
         self.info_map = {}
         for filename in downloads:
             self.info_map[filename] = download_metadata.get(filename, "NOT FOUND unknown to server")
-        if config.writable_cache_or_verbose("Readonly cache, skipping download of (first 5):", repr(downloads[:5]), verbosity=70):
+        if config.writable_cache_or_verbose("Readonly cache, skipping download of (first 5):", repr(downloads[:5]),
+                                            verbosity=70):
             bytes_so_far = 0
             total_files = len(downloads)
             total_bytes = get_total_bytes(self.info_map)
@@ -630,7 +768,8 @@ class FileCacher:
                     if "NOT FOUND" in self.info_map[name]:
                         raise CrdsDownloadError("file is not known to CRDS server.")
                     bytes, path = self.catalog_file_size(name), localpaths[name]
-                    log.info(file_progress("Fetching", name, path, bytes, bytes_so_far, total_bytes, nth_file, total_files))
+                    log.info(
+                        file_progress("Fetching", name, path, bytes, bytes_so_far, total_bytes, nth_file, total_files))
                     self.download(name, path)
                     bytes_so_far += os.stat(path).st_size
                 except Exception as exc:
@@ -658,7 +797,7 @@ class FileCacher:
                 "at CRDS server", srepr(get_crds_server()),
                 "with mode", srepr(config.get_download_mode()),
                 ":", str(exc)) from exc
-        except:  #  mainly for control-c,  catch it and throw it.
+        except:  # mainly for control-c,  catch it and throw it.
             self.remove_file(localpath)
             raise
 
@@ -715,7 +854,8 @@ class FileCacher:
                 stats.increment("bytes", len(data))
                 status = stats.status("bytes")
                 bytes_so_far = " ".join(status[0].split()[:-1])
-                log.verbose("Transferred HTTP", repr(url), bytes_so_far, "/", file_size, "bytes at", status[1], verbosity=20)
+                log.verbose("Transferred HTTP", repr(url), bytes_so_far, "/", file_size, "bytes at", status[1],
+                            verbosity=20)
                 yield data
                 data = infile.read(config.CRDS_DATA_CHUNK_SIZE)
         except Exception as exc:
@@ -725,7 +865,7 @@ class FileCacher:
         finally:
             try:
                 infile.close()
-            except UnboundLocalError:   # maybe the open failed.
+            except UnboundLocalError:  # maybe the open failed.
                 pass
 
     def get_url(self, filename):
@@ -754,8 +894,10 @@ class FileCacher:
         else:
             log.verbose("Skipping sha1sum check since server doesn't know it.")
 
+
 # ==============================================================================
 
+
 def dump_mappings3(pipeline_context, ignore_cache=False, mappings=None, raise_exceptions=True):
     """Given a `pipeline_context`, determine the closure of CRDS mappings for it and
     cache them on the local file system.
@@ -770,6 +912,7 @@ def dump_mappings3(pipeline_context, ignore_cache=False, mappings=None, raise_ex
     mappings = list(reversed(sorted(set(mappings))))
     return FileCacher(pipeline_context, ignore_cache, raise_exceptions).get_local_files(mappings)
 
+
 def dump_mappings(*args, **keys):
     """See dump_mappings3.
 
@@ -777,6 +920,7 @@ def dump_mappings(*args, **keys):
     """
     return dump_mappings3(*args, **keys)[0]
 
+
 def dump_references3(pipeline_context, baserefs=None, ignore_cache=False, raise_exceptions=True):
     """Given a pipeline `pipeline_context` and list of `baserefs` reference
     file basenames,  obtain the set of reference files and cache them on the
@@ -796,6 +940,7 @@ def dump_references3(pipeline_context, baserefs=None, ignore_cache=False, raise_
     baserefs = sorted(set(baserefs))
     return FileCacher(pipeline_context, ignore_cache, raise_exceptions).get_local_files(baserefs)
 
+
 def dump_references(*args, **keys):
     """See dump_references3.
 
@@ -803,6 +948,7 @@ def dump_references(*args, **keys):
     """
     return dump_references3(*args, **keys)[0]
 
+
 def dump_files(pipeline_context=None, files=None, ignore_cache=False, raise_exceptions=True):
     """Unified interface to dump any file in `files`, mapping or reference.
 
@@ -812,8 +958,8 @@ def dump_files(pipeline_context=None, files=None, ignore_cache=False, raise_exce
         pipeline_context = get_default_context()
     if files is None:
         files = get_mapping_names(pipeline_context)
-    mappings = [ os.path.basename(name) for name in files if config.is_mapping(name) ]
-    references = [ os.path.basename(name) for name in files if not config.is_mapping(name) ]
+    mappings = [os.path.basename(name) for name in files if config.is_mapping(name)]
+    references = [os.path.basename(name) for name in files if not config.is_mapping(name)]
     if mappings:
         m_paths, m_downloads, m_bytes = dump_mappings3(
             pipeline_context, mappings=mappings, ignore_cache=ignore_cache, raise_exceptions=raise_exceptions)
@@ -824,7 +970,8 @@ def dump_files(pipeline_context=None, files=None, ignore_cache=False, raise_exce
             pipeline_context, baserefs=references, ignore_cache=ignore_cache, raise_exceptions=raise_exceptions)
     else:
         r_paths, r_downloads, r_bytes = {}, 0, 0
-    return dict(list(m_paths.items())+list(r_paths.items())), m_downloads + r_downloads, m_bytes + r_bytes
+    return dict(list(m_paths.items()) + list(r_paths.items())), m_downloads + r_downloads, m_bytes + r_bytes
+
 
 # =====================================================================================================
 
@@ -837,6 +984,7 @@ def cache_best_references(pipeline_context, header, ignore_cache=False, reftypes
     local_paths = cache_references(pipeline_context, best_refs, ignore_cache)
     return local_paths
 
+
 def cache_references(pipeline_context, bestrefs, ignore_cache=False):
     """Given a CRDS context `pipeline_context` and `bestrefs` dictionary, download missing
     reference files and cache them on the local file system.
@@ -856,6 +1004,7 @@ def cache_references(pipeline_context, bestrefs, ignore_cache=False):
 
     return refs
 
+
 def _get_cache_filelist_and_report_errors(bestrefs):
     """Compute the list of files to download based on the `bestrefs` dictionary,
     skimming off and reporting errors, and raising an exception on the last error seen.
@@ -891,6 +1040,7 @@ def _get_cache_filelist_and_report_errors(bestrefs):
         raise last_error
     return wanted
 
+
 def _squash_unicode_in_bestrefs(bestrefs, localrefs):
     """Given bestrefs dictionariesy `bestrefs` and `localrefs`, make sure
     there are no unicode strings anywhere in the keys or complex
@@ -901,7 +1051,7 @@ def _squash_unicode_in_bestrefs(bestrefs, localrefs):
         if isinstance(refname, tuple):
             refs[str(filetype)] = tuple([str(localrefs[name]) for name in refname])
         elif isinstance(refname, dict):
-            refs[str(filetype)] = { name : str(localrefs[name]) for name in refname }
+            refs[str(filetype)] = {name: str(localrefs[name]) for name in refname}
         elif isinstance(refname, str):
             if "NOT FOUND" in refname:
                 refs[str(filetype)] = str(refname)
@@ -912,6 +1062,7 @@ def _squash_unicode_in_bestrefs(bestrefs, localrefs):
                 "Unhandled bestrefs return value type for", srepr(filetype))
     return refs
 
+
 # =====================================================================================================
 
 # These functions are deprecated and only work when the full CRDS library is installed,  and only for
@@ -926,11 +1077,12 @@ def cache_best_references_for_dataset(pipeline_context, dataset,
     header = get_minimum_header(pipeline_context, dataset, ignore_cache)
     return cache_best_references(pipeline_context, header, ignore_cache)
 
+
 def get_minimum_header(context, dataset, ignore_cache=False):
     """Given a `dataset` and a `context`,  extract relevant header
     information from the `dataset`.
     """
     import crds
     dump_mappings(context, ignore_cache=ignore_cache)
-    ctx = crds.get_pickled_mapping(context)   # reviewed
+    ctx = crds.get_pickled_mapping(context)  # reviewed
     return ctx.get_minimum_header(dataset)
