@@ -28,7 +28,7 @@ def _get_crds_ini_parser():
     """Load and return the environment from the CRDS rc file."""
     global CRDS_INI_PARSER
     if CRDS_INI_PARSER is None:
-        parser = configparser.SafeConfigParser()
+        parser = configparser.ConfigParser()
         with log.warn_on_exception("Failed reading CRDS rc file"):
             ini_path = _get_crds_ini_path()
             if os.path.exists(ini_path):
@@ -661,7 +661,7 @@ def get_crds_env_context():
     context = os.environ.get("CRDS_CONTEXT", None)
     if context:
         assert is_context_spec(context), \
-            "Only set CRDS_CONTEXT to a literal or symbolic context (.pmap), e.g. jwst_0042.pmap,  jwst-2014-10-15T00:15:21, jwst-operational,  not " + repr(context)
+            "Only set CRDS_CONTEXT to a literal or symbolic context (.pmap), e.g. jwst_0042.pmap,  jwst-2014-10-15T00:15:21, jwst-latest,  not " + repr(context)
     return context
 
 CRDS_IGNORE_MAPPING_CHECKSUM = BooleanConfigItem("CRDS_IGNORE_MAPPING_CHECKSUM", False,
@@ -781,6 +781,12 @@ def get_client_retry_delay_seconds():
     """Return the integer number of seconds CRDS should wait between retrying failed network transactions."""
     return CLIENT_RETRY_DELAY_SECONDS.get()
 
+CLIENT_TIMEOUT = IntConfigItem(
+    "CRDS_CLIENT_TIMEOUT_SECONDS", 3600, "Seconds to wait for a CRDS network request to complete.")
+
+def get_client_timeout_seconds():
+    return CLIENT_TIMEOUT.get()
+
 def enable_retries(retry_count=20, delay_seconds=10):
     """Set reasonable defaults for CRDS retries"""
     CLIENT_RETRY_COUNT.set(retry_count)
@@ -793,6 +799,8 @@ def disable_retries():
 
 # -------------------------------------------------------------------------------------
 
+# -------------------------------------------------------------------------------------
+
 CRDS_DEFAULT_SERVERS = {
     "hst" : "https://hst-crds.stsci.edu",
     "jwst" : "https://jwst-crds.stsci.edu",
@@ -1203,7 +1211,7 @@ CONTEXT_RE_STR = (
                 r"(?P<date>" +
                     r"(" + CONTEXT_DATETIME_RE_STR + r")" +
                         r"|" +
-                    r"(" + "edit|operational|versions" + r")" +
+                    r"(" + "edit|operational|latest|build|versions" + r")" +
                 r")" +
             r")" +
         r")" +
@@ -1223,7 +1231,7 @@ PIPELINE_CONTEXT_RE_STR = (
                 r"(" +
                     r"(?P<date>" + CONTEXT_DATETIME_RE_STR + r")" +
                         "|" +
-                    r"(?P<context_tag>" + "edit|operational|versions" + r")" +
+                    r"(?P<context_tag>" + "edit|operational|latest|build|versions" + r")" +
                 r")" +
             r")" +
         r")"
@@ -1340,7 +1348,16 @@ def is_mapping_spec(mapping):
     >>> is_mapping_spec("hst-cos-deadtab-edit")
     True
 
-    >>> is_mapping_spec("jwst-operational")
+    >>> is_mapping_spec("jwst-latest")
+    True
+
+    >>> is_mapping_spec("latest")
+    True
+
+    >>> is_mapping_spec("jwst-build")
+    True
+
+    >>> is_mapping_spec("build")
     True
 
     >>> is_mapping_spec("hst-foo")
@@ -1349,7 +1366,7 @@ def is_mapping_spec(mapping):
     >>> is_mapping_spec("hst_wfc3_0001.imap")
     True
     """
-    return is_mapping(mapping) or (isinstance(mapping, str) and bool(CONTEXT_RE.match(mapping)))
+    return is_mapping(mapping) or (isinstance(mapping, str) and bool(CONTEXT_RE.match(mapping))) or mapping in ["latest", "build"]
 
 def is_context(mapping):
     """Return True IFF `mapping` has an extension indicating a CRDS CONTEXT, i.e. .pmap."""
@@ -1383,6 +1400,8 @@ def is_context_spec(mapping):
     >>> is_context_spec("hst-acs-2040-01-29T12:00:00")
     False
     """
+    if mapping in ["latest", "build"]:
+        return True
     return is_context(mapping) or (isinstance(mapping, str) and bool(PIPELINE_CONTEXT_RE.match(mapping)))
 
 def is_date_based_mapping_spec(mapping):
