@@ -55,7 +55,7 @@ def get_datamodels():
     if MODEL is None:
         with log.error_on_exception(
                 "Failed constructing basic JWST DataModel"):
-            MODEL = datamodels.DataModel()
+            MODEL = datamodels.JwstDataModel()
     return datamodels
 
 # =============================================================================
@@ -99,7 +99,7 @@ def match_context_key(key):
 
 # =======================================================================
 
-REF_EXT_RE = re.compile(r"\.fits|\.r\dh$")
+REF_EXT_RE = re.compile(r"\.(fits|r\dh)$")
 
 @utils.cached
 def get_file_properties(filename):
