@@ -350,6 +350,9 @@ def ref_properties_from_header(filename):
             "Can't identify instrument of", repr(name), ":", str(exc)) from exc
     try:
         filekind = header.get('ROMAN.META.REFTYPE', 'UNDEFINED').lower()
+        # Not all headers are roman datamodels.
+        if filekind == 'undefined':
+            filekind = header.get('META.REFTYPE', 'UNDEFINED').lower()
         assert filekind in FILEKINDS, "Invalid file type " + repr(filekind)
     except Exception as exc:
         raise exceptions.CrdsNamingError("Can't identify ROMAN.META.REFTYPE of", repr(name))
@@ -443,9 +446,8 @@ def reference_keys_to_dataset_keys(rmapping, header):
     >>> config.ALLOW_BAD_USEAFTER.reset()
     >>> reference_keys_to_dataset_keys( \
     namedtuple('x', ['reference_to_dataset', 'filename'])({}, 'secret_code_file.txt'), \
-    {'ROMAN.META.OBSERVATION.DATE' : '1879-03-14', \
-     'ROMAN.META.OBSERVATION.TIME' : '12:34:56'})
-    {'ROMAN.META.OBSERVATION.DATE': '1879-03-14', 'ROMAN.META.OBSERVATION.TIME': '12:34:56', 'ROMAN.META.SUBARRAY.NAME': 'UNDEFINED', 'ROMAN.META.EXPOSURE.TYPE': 'UNDEFINED'}
+    {'ROMAN.META.EXPOSURE.START_TIME' : '1879-03-14 12:34:56'})
+    {'ROMAN.META.EXPOSURE.START_TIME': '1879-03-14 12:34:56', 'ROMAN.META.SUBARRAY.NAME': 'UNDEFINED', 'ROMAN.META.EXPOSURE.TYPE': 'UNDEFINED'}
 
     ==================================================
     Test setting DATE/TIME with no USEAFTER, but allowed "bad use after".
@@ -454,7 +456,7 @@ def reference_keys_to_dataset_keys(rmapping, header):
     >>> config.ALLOW_BAD_USEAFTER.set("1")
     False
     >>> reference_keys_to_dataset_keys(namedtuple('x', ['reference_to_dataset', 'filename'])({}, 'secret_code_file.txt'), {})
-    {'ROMAN.META.SUBARRAY.NAME': 'UNDEFINED', 'ROMAN.META.EXPOSURE.TYPE': 'UNDEFINED', 'ROMAN.META.OBSERVATION.DATE': '1900-01-01', 'ROMAN.META.OBSERVATION.TIME': '00:00:00'}
+    {'ROMAN.META.SUBARRAY.NAME': 'UNDEFINED', 'ROMAN.META.EXPOSURE.TYPE': 'UNDEFINED', 'ROMAN.META.EXPOSURE.START_TIME': '1900-01-01 00:00:00'}
 
     ==================================================
     Test setting DATE/TIME from USEAFTER.
@@ -463,31 +465,29 @@ def reference_keys_to_dataset_keys(rmapping, header):
     >>> config.ALLOW_BAD_USEAFTER.set("1")
     False
     >>> reference_keys_to_dataset_keys(namedtuple('x', ['reference_to_dataset', 'filename'])({}, 'secret_code_file.txt'), \
-    {'ROMAN.META.USEAFTER' : '1770-12-01T01:23:45', \
-     'ROMAN.META.OBSERVATION.DATE' : '1879-03-14', \
-     'ROMAN.META.OBSERVATION.TIME' : '12:34:56'})
-    {'ROMAN.META.USEAFTER': '1770-12-01T01:23:45', 'ROMAN.META.OBSERVATION.DATE': '1770-12-01', 'ROMAN.META.OBSERVATION.TIME': '01:23:45', 'ROMAN.META.SUBARRAY.NAME': 'UNDEFINED', 'ROMAN.META.EXPOSURE.TYPE': 'UNDEFINED'}
+    {'ROMAN.META.USEAFTER' : '1770-12-01 01:23:45', 'ROMAN.META.EXPOSURE.START_TIME' : '1879-03-14 12:34:56'})
+    {'ROMAN.META.USEAFTER': '1770-12-01 01:23:45', 'ROMAN.META.EXPOSURE.START_TIME': '1770-12-01 01:23:45', 'ROMAN.META.SUBARRAY.NAME': 'UNDEFINED', 'ROMAN.META.EXPOSURE.TYPE': 'UNDEFINED'}
 
     ==================================================
     Test bad formatted USEAFTER.
 
     >>> config.ALLOW_BAD_USEAFTER.reset()
     >>> reference_keys_to_dataset_keys(namedtuple('x', ['reference_to_dataset', 'filename'])({}, 'secret_code_file.txt'), \
-    {'ROMAN.META.USEAFTER' : 'bad user after', \
-     'ROMAN.META.OBSERVATION.DATE' : '1879-03-14', \
-     'ROMAN.META.OBSERVATION.TIME' : '12:34:56'})
+    {'ROMAN.META.USEAFTER' : 'bad user after', 'ROMAN.META.EXPOSURE.START_TIME' : '1879-03-14 12:34:56'})
     Traceback (most recent call last):
     ...
     crds.core.exceptions.InvalidUseAfterFormat: Bad USEAFTER time format = 'bad user after'
     """
     header = dict(header)
-
+    # if parameter reference file (contains "-"), there is no "ROMAN" prefix
+    paramfile = True if len(rmapping.name.split("-")) > 1 else False
+    prefix = "META" if paramfile is True else "ROMAN.META"
     # Basic common pattern translations
     translations = {
-        "ROMAN.META.EXPOSURE.P_EXPTYPE" : "ROMAN.META.EXPOSURE.TYPE",
-        "ROMAN.META.INSTRUMENT.P_DETECTOR"  : "ROMAN.META.INSTRUMENT.DETECTOR",
+        f"{prefix}.EXPOSURE.P_EXPTYPE": f"{prefix}.EXPOSURE.TYPE",
+        f"{prefix}.INSTRUMENT.P_DETECTOR"  : f"{prefix}.INSTRUMENT.DETECTOR",
+        f"{prefix}.INSTRUMENT.P_OPTICAL_ELEMENT": f"{prefix}.INSTRUMENT.OPTICAL_ELEMENT",
     }
-
     # Rmap header reference_to_dataset field tranlations,  can override basic!
     try:
         translations.update(rmapping.reference_to_dataset)
@@ -523,22 +523,21 @@ def reference_keys_to_dataset_keys(rmapping, header):
                          "to value of", repr(rkey), "=", repr(rval))
                 header[dkey] = rval
 
-    if "ROMAN.META.SUBARRAY.NAME" not in header:
-        header["ROMAN.META.SUBARRAY.NAME"] = "UNDEFINED"
-
-    if "ROMAN.META.EXPOSURE.TYPE" not in header:
-        header["ROMAN.META.EXPOSURE.TYPE"] = "UNDEFINED"
+    if f"{prefix}.SUBARRAY.NAME" not in header:
+        header[f"{prefix}.SUBARRAY.NAME"] = "UNDEFINED"
+    if f"{prefix}.EXPOSURE.TYPE" not in header:
+        header[f"{prefix}.EXPOSURE.TYPE"] = "UNDEFINED"
 
     # If USEAFTER is defined,  or we're configured to fake it...
     #   don't invent one if its missing and we're not faking it.
-    if "ROMAN.META.USEAFTER" in header or config.ALLOW_BAD_USEAFTER:
+    if f"{prefix}.USEAFTER" in header or config.ALLOW_BAD_USEAFTER:
 
         # Identify this as best as possible,
-        filename = header.get("ROMAN.META.FILENAME", None) or rmapping.filename
+        filename = header.get(f"{prefix}.FILENAME", None) or rmapping.filename
 
         reformatted = timestamp.reformat_useafter(filename, header).split()
-        header["ROMAN.META.OBSERVATION.DATE"] = reformatted[0]
-        header["ROMAN.META.OBSERVATION.TIME"] = reformatted[1]
+        dt_string = f"{reformatted[0]} {reformatted[1]}"
+        header["ROMAN.META.EXPOSURE.START_TIME"] = dt_string
 
     log.verbose("reference_to_dataset output header:\n", log.PP(header), verbosity=80)
 
@@ -688,6 +687,19 @@ def get_cross_strapped_pairs(header):
     """
     return []
 
+
+def dataset_to_ref_header(parameters):
+    """Temporary workaround (hopefully) for prepending archive dataset header keys that start with 'META' to start with 'ROMAN.META' instead."""
+    ref_headers = dict()
+    for key, val in parameters.items():
+        parts = key.split('.')
+        if parts[0].upper() == 'META':
+            param = "ROMAN." + '.'.join(parts[:])
+        else:
+            param = key
+        ref_headers[param] = val
+    return ref_headers
+
 # ============================================================================
 
 def test():
